@@ -8,11 +8,7 @@
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   GNU General Public License for more details.  */
 
 #include "cvs.h"
 #include "getline.h"
@@ -29,23 +25,26 @@ static int setting_tedit;
 static int setting_tunedit;
 static int setting_tcommit;
 
-static int onoff_fileproc PROTO ((struct file_info *finfo));
+static int onoff_fileproc PROTO ((void *callerdat, struct file_info *finfo));
 
 static int
-onoff_fileproc (finfo)
+onoff_fileproc (callerdat, finfo)
+    void *callerdat;
     struct file_info *finfo;
 {
     fileattr_set (finfo->file, "_watched", turning_on ? "" : NULL);
     return 0;
 }
 
-static int onoff_filesdoneproc PROTO ((int, char *, char *));
+static int onoff_filesdoneproc PROTO ((void *, int, char *, char *, List *));
 
 static int
-onoff_filesdoneproc (err, repository, update_dir)
+onoff_filesdoneproc (callerdat, err, repository, update_dir, entries)
+    void *callerdat;
     int err;
     char *repository;
     char *update_dir;
+    List *entries;
 {
     if (setting_default)
 	fileattr_set (NULL, "_watched", turning_on ? "" : NULL);
@@ -61,14 +60,17 @@ watch_onoff (argc, argv)
     int local = 0;
     int err;
 
-    optind = 1;
-    while ((c = getopt (argc, argv, "l")) != -1)
+    optind = 0;
+    while ((c = getopt (argc, argv, "+lR")) != -1)
     {
 	switch (c)
 	{
 	    case 'l':
 		local = 1;
 		break;
+	    case 'R':
+		local = 0;
+		break;
 	    case '?':
 	    default:
 		usage (watch_usage);
@@ -79,7 +81,7 @@ watch_onoff (argc, argv)
     argv += optind;
 
 #ifdef CLIENT_SUPPORT
-    if (client_active)
+    if (current_parsed_root->isremote)
     {
 	start_server ();
 
@@ -87,11 +89,8 @@ watch_onoff (argc, argv)
 
 	if (local)
 	    send_arg ("-l");
+	send_files (argc, argv, local, 0, SEND_NO_CONTENTS);
 	send_file_names (argc, argv, SEND_EXPAND_WILD);
-	/* FIXME:  We shouldn't have to send current files, but I'm not sure
-	   whether it works.  So send the files --
-	   it's slower but it works.  */
-	send_files (argc, argv, local, 0);
 	send_to_server (turning_on ? "watch-on\012" : "watch-off\012", 0);
 	return get_responses_and_close ();
     }
@@ -99,14 +98,14 @@ watch_onoff (argc, argv)
 
     setting_default = (argc <= 0);
 
-    lock_tree_for_write (argc, argv, local, 0);
+    lock_tree_for_write (argc, argv, local, W_LOCAL, 0);
 
     err = start_recursion (onoff_fileproc, onoff_filesdoneproc,
-			   (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL,
+			   (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
 			   argc, argv, local, W_LOCAL, 0, 0, (char *)NULL,
-			   0, 0);
+			   0);
 
-    lock_tree_cleanup ();
+    Lock_Cleanup ();
     return err;
 }
 
@@ -128,10 +127,11 @@ watch_off (argc, argv)
     return watch_onoff (argc, argv);
 }
 
-static int dummy_fileproc PROTO ((struct file_info *finfo));
+static int dummy_fileproc PROTO ((void *callerdat, struct file_info *finfo));
 
 static int
-dummy_fileproc (finfo)
+dummy_fileproc (callerdat, finfo)
+    void *callerdat;
     struct file_info *finfo;
 {
     /* This is a pretty hideous hack, but the gist of it is that recurse.c
@@ -140,7 +140,7 @@ dummy_fileproc (finfo)
     return 0;
 }
 
-static int ncheck_fileproc PROTO ((struct file_info *finfo));
+static int ncheck_fileproc PROTO ((void *callerdat, struct file_info *finfo));
 
 /* Check for and process notifications.  Local only.  I think that doing
    this as a fileproc is the only way to catch all the
@@ -149,7 +149,8 @@ static int ncheck_fileproc PROTO ((struct file_info *finfo));
    processed the directory.  */
 
 static int
-ncheck_fileproc (finfo)
+ncheck_fileproc (callerdat, finfo)
+    void *callerdat;
     struct file_info *finfo;
 {
     int notif_type;
@@ -165,7 +166,7 @@ ncheck_fileproc (finfo)
     /* We send notifications even if noexec.  I'm not sure which behavior
        is most sensible.  */
 
-    fp = fopen (CVSADM_NOTIFY, "r");
+    fp = CVS_FOPEN (CVSADM_NOTIFY, "r");
     if (fp == NULL)
     {
 	if (!existence_error (errno))
@@ -212,7 +213,7 @@ ncheck_fileproc (finfo)
     if (fclose (fp) < 0)
 	error (0, errno, "cannot close %s", CVSADM_NOTIFY);
 
-    if (unlink (CVSADM_NOTIFY) < 0)
+    if ( CVS_UNLINK (CVSADM_NOTIFY) < 0)
 	error (0, errno, "cannot remove %s", CVSADM_NOTIFY);
 
     return 0;
@@ -234,7 +235,7 @@ send_notifications (argc, argv, local)
     /* OK, we've done everything which needs to happen on the client side.
        Now we can try to contact the server; if we fail, then the
        notifications stay in CVSADM_NOTIFY to be sent next time.  */
-    if (client_active)
+    if (current_parsed_root->isremote)
     {
 	if (strcmp (command_name, "release") != 0)
 	{
@@ -243,9 +244,9 @@ send_notifications (argc, argv, local)
 	}
 
 	err += start_recursion (dummy_fileproc, (FILESDONEPROC) NULL,
-				(DIRENTPROC) NULL, (DIRLEAVEPROC) NULL,
+				(DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
 				argc, argv, local, W_LOCAL, 0, 0, (char *)NULL,
-				0, 0);
+				0);
 
 	send_to_server ("noop\012", 0);
 	if (strcmp (command_name, "release") == 0)
@@ -258,20 +259,21 @@ send_notifications (argc, argv, local)
     {
 	/* Local.  */
 
-	lock_tree_for_write (argc, argv, local, 0);
+	lock_tree_for_write (argc, argv, local, W_LOCAL, 0);
 	err += start_recursion (ncheck_fileproc, (FILESDONEPROC) NULL,
-				(DIRENTPROC) NULL, (DIRLEAVEPROC) NULL,
+				(DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
 				argc, argv, local, W_LOCAL, 0, 0, (char *)NULL,
-				0, 0);
-	lock_tree_cleanup ();
+				0);
+	Lock_Cleanup ();
     }
     return err;
 }
 
-static int edit_fileproc PROTO ((struct file_info *finfo));
+static int edit_fileproc PROTO ((void *callerdat, struct file_info *finfo));
 
 static int
-edit_fileproc (finfo)
+edit_fileproc (callerdat, finfo)
+    void *callerdat;
     struct file_info *finfo;
 {
     FILE *fp;
@@ -282,11 +284,25 @@ edit_fileproc (finfo)
     if (noexec)
 	return 0;
 
+    /* This is a somewhat screwy way to check for this, because it
+       doesn't help errors other than the nonexistence of the file
+       (e.g. permissions problems).  It might be better to rearrange
+       the code so that CVSADM_NOTIFY gets written only after the
+       various actions succeed (but what if only some of them
+       succeed).  */
+    if (!isfile (finfo->file))
+    {
+	error (0, 0, "no such file %s; ignored", finfo->fullname);
+	return 0;
+    }
+
     fp = open_file (CVSADM_NOTIFY, "a");
 
     (void) time (&now);
     ascnow = asctime (gmtime (&now));
     ascnow[24] = '\0';
+    /* Fix non-standard format.  */
+    if (ascnow[8] == '0') ascnow[8] = ' ';
     fprintf (fp, "E%s\t%s GMT\t%s\t%s\t", finfo->file,
 	     ascnow, hostname, CurDir);
     if (setting_tedit)
@@ -313,19 +329,10 @@ edit_fileproc (finfo)
        copy so that if the user removes the working file, then restores it
        with "cvs update" (which clears _editors but does not update
        CVSADM_BASE), then a future "cvs edit" can still win.  */
-    /* Could save a system call by only calling mkdir if trying to create
-       the output file fails.  But copy_file isn't set up to facilitate
-       that.  */
-    if (CVS_MKDIR (CVSADM_BASE, 0777) < 0)
-    {
-		if (errno != EEXIST
-#ifdef EACCESS
-		    /* OS/2; see longer comment in client.c.  */
-		    && errno != EACCESS
-#endif
-		    )
-	    error (1, errno, "cannot mkdir %s", CVSADM_BASE);
-    }
+    /* Could save a system call by only calling mkdir_if_needed if
+       trying to create the output file fails.  But copy_file isn't
+       set up to facilitate that.  */
+    mkdir_if_needed (CVSADM_BASE);
     basefilename = xmalloc (10 + sizeof CVSADM_BASE + strlen (finfo->file));
     strcpy (basefilename, CVSADM_BASE);
     strcat (basefilename, "/");
@@ -333,15 +340,25 @@ edit_fileproc (finfo)
     copy_file (finfo->file, basefilename);
     free (basefilename);
 
+    {
+	Node *node;
+
+	node = findnode_fn (finfo->entries, finfo->file);
+	if (node != NULL)
+	    base_register (finfo, ((Entnode *) node->data)->version);
+    }
+
     return 0;
 }
 
 static const char *const edit_usage[] =
 {
-    "Usage: %s %s [-l] [files...]\n",
+    "Usage: %s %s [-lR] [files...]\n",
     "-l: Local directory only, not recursive\n",
+    "-R: Process directories recursively\n",
     "-a: Specify what actions for temporary watch, one of\n",
-    "    edit,unedit,commit.all,none\n",
+    "    edit,unedit,commit,all,none\n",
+    "(Specify the --help global option for a list of other help options)\n",
     NULL
 };
 
@@ -362,14 +379,17 @@ edit (argc, argv)
     setting_tedit = 0;
     setting_tunedit = 0;
     setting_tcommit = 0;
-    optind = 1;
-    while ((c = getopt (argc, argv, "la:")) != -1)
+    optind = 0;
+    while ((c = getopt (argc, argv, "+lRa:")) != -1)
     {
 	switch (c)
 	{
 	    case 'l':
 		local = 1;
 		break;
+	    case 'R':
+		local = 0;
+		break;
 	    case 'a':
 		a_omitted = 0;
 		if (strcmp (optarg, "edit") == 0)
@@ -409,22 +429,32 @@ edit (argc, argv)
 	setting_tcommit = 1;
     }
 
+    if (strpbrk (hostname, "+,>;=\t\n") != NULL)
+	error (1, 0,
+	       "host name (%s) contains an invalid character (+,>;=\\t\\n)",
+	       hostname);
+    if (strpbrk (CurDir, "+,>;=\t\n") != NULL)
+	error (1, 0,
+"current directory (%s) contains an invalid character (+,>;=\\t\\n)",
+	       CurDir);
+
     /* No need to readlock since we aren't doing anything to the
        repository.  */
     err = start_recursion (edit_fileproc, (FILESDONEPROC) NULL,
-			   (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL,
+			   (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
 			   argc, argv, local, W_LOCAL, 0, 0, (char *)NULL,
-			   0, 0);
+			   0);
 
     err += send_notifications (argc, argv, local);
 
     return err;
 }
 
-static int unedit_fileproc PROTO ((struct file_info *finfo));
+static int unedit_fileproc PROTO ((void *callerdat, struct file_info *finfo));
 
 static int
-unedit_fileproc (finfo)
+unedit_fileproc (callerdat, finfo)
+    void *callerdat;
     struct file_info *finfo;
 {
     FILE *fp;
@@ -465,6 +495,8 @@ unedit_fileproc (finfo)
     (void) time (&now);
     ascnow = asctime (gmtime (&now));
     ascnow[24] = '\0';
+    /* Fix non-standard format.  */
+    if (ascnow[8] == '0') ascnow[8] = ' ';
     fprintf (fp, "U%s\t%s GMT\t%s\t%s\t\n", finfo->file,
 	     ascnow, hostname, CurDir);
 
@@ -477,10 +509,72 @@ unedit_fileproc (finfo)
 		   CVSADM_NOTIFY);
     }
 
+    /* Now update the revision number in CVS/Entries from CVS/Baserev.
+       The basic idea here is that we are reverting to the revision
+       that the user edited.  If we wanted "cvs update" to update
+       CVS/Base as we go along (so that an unedit could revert to the
+       current repository revision), we would need:
+
+       update (or all send_files?) (client) needs to send revision in
+       new Entry-base request.  update (server/local) needs to check
+       revision against repository and send new Update-base response
+       (like Update-existing in that the file already exists.  While
+       we are at it, might try to clean up the syntax by having the
+       mode only in a "Mode" response, not in the Update-base itself).  */
+    {
+	char *baserev;
+	Node *node;
+	Entnode *entdata;
+
+	baserev = base_get (finfo);
+	node = findnode_fn (finfo->entries, finfo->file);
+	/* The case where node is NULL probably should be an error or
+	   something, but I don't want to think about it too hard right
+	   now.  */
+	if (node != NULL)
+	{
+	    entdata = (Entnode *) node->data;
+	    if (baserev == NULL)
+	    {
+		/* This can only happen if the CVS/Baserev file got
+		   corrupted.  We suspect it might be possible if the
+		   user interrupts CVS, although I haven't verified
+		   that.  */
+		error (0, 0, "%s not mentioned in %s", finfo->fullname,
+		       CVSADM_BASEREV);
+
+		/* Since we don't know what revision the file derives from,
+		   keeping it around would be asking for trouble.  */
+		if (unlink_file (finfo->file) < 0)
+		    error (0, errno, "cannot remove %s", finfo->fullname);
+
+		/* This is cheesy, in a sense; why shouldn't we do the
+		   update for the user?  However, doing that would require
+		   contacting the server, so maybe this is OK.  */
+		error (0, 0, "run update to complete the unedit");
+		return 0;
+	    }
+	    Register (finfo->entries, finfo->file, baserev, entdata->timestamp,
+		      entdata->options, entdata->tag, entdata->date,
+		      entdata->conflict);
+	}
+	free (baserev);
+	base_deregister (finfo);
+    }
+
     xchmod (finfo->file, 0);
     return 0;
 }
 
+static const char *const unedit_usage[] =
+{
+    "Usage: %s %s [-lR] [files...]\n",
+    "-l: Local directory only, not recursive\n",
+    "-R: Process directories recursively\n",
+    "(Specify the --help global option for a list of other help options)\n",
+    NULL
+};
+
 int
 unedit (argc, argv)
     int argc;
@@ -491,19 +585,22 @@ unedit (argc, argv)
     int err;
 
     if (argc == -1)
-	usage (edit_usage);
+	usage (unedit_usage);
 
-    optind = 1;
-    while ((c = getopt (argc, argv, "l")) != -1)
+    optind = 0;
+    while ((c = getopt (argc, argv, "+lR")) != -1)
     {
 	switch (c)
 	{
 	    case 'l':
 		local = 1;
 		break;
+	    case 'R':
+		local = 0;
+		break;
 	    case '?':
 	    default:
-		usage (edit_usage);
+		usage (unedit_usage);
 		break;
 	}
     }
@@ -513,9 +610,9 @@ unedit (argc, argv)
     /* No need to readlock since we aren't doing anything to the
        repository.  */
     err = start_recursion (unedit_fileproc, (FILESDONEPROC) NULL,
-			   (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL,
+			   (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
 			   argc, argv, local, W_LOCAL, 0, 0, (char *)NULL,
-			   0, 0);
+			   0);
 
     err += send_notifications (argc, argv, local);
 
@@ -551,14 +648,14 @@ editor_set (filename, editor, val)
 
     edlist = fileattr_get0 (filename, "_editors");
     newlist = fileattr_modify (edlist, editor, val, '>', ',');
-    if (edlist != NULL)
-	free (edlist);
     /* If the attributes is unchanged, don't rewrite the attribute file.  */
     if (!((edlist == NULL && newlist == NULL)
 	  || (edlist != NULL
 	      && newlist != NULL
 	      && strcmp (edlist, newlist) == 0)))
 	fileattr_set (filename, "_editors", newlist);
+    if (edlist != NULL)
+	free (edlist);
     if (newlist != NULL)
 	free (newlist);
 }
@@ -647,6 +744,9 @@ notify_proc (repository, filter)
     return (pclose (pipefp));
 }
 
+/* FIXME: this function should have a way to report whether there was
+   an error so that server.c can know whether to report Notified back
+   to the client.  */
 void
 notify_do (type, filename, who, val, watches, repository)
     int type;
@@ -668,6 +768,11 @@ notify_do (type, filename, who, val, watches, repository)
     switch (type)
     {
 	case 'E':
+	    if (strpbrk (val, ",>;=\n") != NULL)
+	    {
+		error (0, 0, "invalid character in editor value");
+		return;
+	    }
 	    editor_set (filename, who, val);
 	    break;
 	case 'U':
@@ -768,16 +873,16 @@ notify_do (type, filename, who, val, watches, repository)
 	    size_t line_len = 0;
 
 	    args.notifyee = NULL;
-	    usersname = xmalloc (strlen (CVSroot)
+	    usersname = xmalloc (strlen (current_parsed_root->directory)
 				 + sizeof CVSROOTADM
 				 + sizeof CVSROOTADM_USERS
 				 + 20);
-	    strcpy (usersname, CVSroot);
+	    strcpy (usersname, current_parsed_root->directory);
 	    strcat (usersname, "/");
 	    strcat (usersname, CVSROOTADM);
 	    strcat (usersname, "/");
 	    strcat (usersname, CVSROOTADM_USERS);
-	    fp = fopen (usersname, "r");
+	    fp = CVS_FOPEN (usersname, "r");
 	    if (fp == NULL && !existence_error (errno))
 		error (0, errno, "cannot read %s", usersname);
 	    if (fp != NULL)
@@ -789,7 +894,14 @@ notify_do (type, filename, who, val, watches, repository)
 		    {
 			char *cp;
 			args.notifyee = xstrdup (line + len + 1);
-			cp = strchr (args.notifyee, ':');
+
+                        /* There may or may not be more
+                           colon-separated fields added to this in the
+                           future; in any case, we ignore them right
+                           now, and if there are none we make sure to
+                           chop off the final newline, if any. */
+			cp = strpbrk (args.notifyee, ":\n");
+
 			if (cp != NULL)
 			    *cp = '\0';
 			break;
@@ -801,7 +913,8 @@ notify_do (type, filename, who, val, watches, repository)
 		    error (0, errno, "cannot close %s", usersname);
 	    }
 	    free (usersname);
-	    free (line);
+	    if (line != NULL)
+		free (line);
 
 	    if (args.notifyee == NULL)
 	    {
@@ -871,7 +984,7 @@ notify_check (repository, update_dir)
     /* We send notifications even if noexec.  I'm not sure which behavior
        is most sensible.  */
 
-    fp = fopen (CVSADM_NOTIFY, "r");
+    fp = CVS_FOPEN (CVSADM_NOTIFY, "r");
     if (fp == NULL)
     {
 	if (!existence_error (errno))
@@ -897,7 +1010,8 @@ notify_check (repository, update_dir)
 
 	client_notify (repository, update_dir, filename, notif_type, val);
     }
-
+    if (line)
+	free (line);
     if (ferror (fp))
 	error (0, errno, "cannot read %s", CVSADM_NOTIFY);
     if (fclose (fp) < 0)
@@ -911,14 +1025,18 @@ notify_check (repository, update_dir)
 
 static const char *const editors_usage[] =
 {
-    "Usage: %s %s [files...]\n",
+    "Usage: %s %s [-lR] [files...]\n",
+    "\t-l\tProcess this directory only (not recursive).\n",
+    "\t-R\tProcess directories recursively.\n",
+    "(Specify the --help global option for a list of other help options)\n",
     NULL
 };
 
-static int editors_fileproc PROTO ((struct file_info *finfo));
+static int editors_fileproc PROTO ((void *callerdat, struct file_info *finfo));
 
 static int
-editors_fileproc (finfo)
+editors_fileproc (callerdat, finfo)
+    void *callerdat;
     struct file_info *finfo;
 {
     char *them;
@@ -928,29 +1046,29 @@ editors_fileproc (finfo)
     if (them == NULL)
 	return 0;
 
-    fputs (finfo->fullname, stdout);
+    cvs_output (finfo->fullname, 0);
 
     p = them;
     while (1)
     {
-	putc ('\t', stdout);
+	cvs_output ("\t", 1);
 	while (*p != '>' && *p != '\0')
-	    putc (*p++, stdout);
+	    cvs_output (p++, 1);
 	if (*p == '\0')
 	{
 	    /* Only happens if attribute is misformed.  */
-	    putc ('\n', stdout);
+	    cvs_output ("\n", 1);
 	    break;
 	}
 	++p;
-	putc ('\t', stdout);
+	cvs_output ("\t", 1);
 	while (1)
 	{
 	    while (*p != '+' && *p != ',' && *p != '\0')
-		putc (*p++, stdout);
+		cvs_output (p++, 1);
 	    if (*p == '\0')
 	    {
-		putc ('\n', stdout);
+		cvs_output ("\n", 1);
 		goto out;
 	    }
 	    if (*p == ',')
@@ -959,11 +1077,12 @@ editors_fileproc (finfo)
 		break;
 	    }
 	    ++p;
-	    putc ('\t', stdout);
+	    cvs_output ("\t", 1);
 	}
-	putc ('\n', stdout);
+	cvs_output ("\n", 1);
     }
   out:;
+    free (them);
     return 0;
 }
 
@@ -978,14 +1097,17 @@ editors (argc, argv)
     if (argc == -1)
 	usage (editors_usage);
 
-    optind = 1;
-    while ((c = getopt (argc, argv, "l")) != -1)
+    optind = 0;
+    while ((c = getopt (argc, argv, "+lR")) != -1)
     {
 	switch (c)
 	{
 	    case 'l':
 		local = 1;
 		break;
+	    case 'R':
+		local = 0;
+		break;
 	    case '?':
 	    default:
 		usage (editors_usage);
@@ -996,25 +1118,22 @@ editors (argc, argv)
     argv += optind;
 
 #ifdef CLIENT_SUPPORT
-    if (client_active)
+    if (current_parsed_root->isremote)
     {
 	start_server ();
 	ign_setup ();
 
 	if (local)
 	    send_arg ("-l");
+	send_files (argc, argv, local, 0, SEND_NO_CONTENTS);
 	send_file_names (argc, argv, SEND_EXPAND_WILD);
-	/* FIXME:  We shouldn't have to send current files, but I'm not sure
-	   whether it works.  So send the files --
-	   it's slower but it works.  */
-	send_files (argc, argv, local, 0);
 	send_to_server ("editors\012", 0);
 	return get_responses_and_close ();
     }
 #endif /* CLIENT_SUPPORT */
 
     return start_recursion (editors_fileproc, (FILESDONEPROC) NULL,
-			    (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL,
+			    (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
 			    argc, argv, local, W_LOCAL, 0, 1, (char *)NULL,
-			    0, 0);
+			    0);
 }
