@@ -106,25 +106,25 @@ Name_Repository (dir, update_dir)
      * one by tacking on the CVSROOT environment variable. If the CVSROOT
      * environment variable is not set, die now.
      */
-    if (strcmp (repos, "..") == 0 || strncmp (repos, "../", 3) == 0)
-    {
-	error (0, 0, "in directory %s:", xupdate_dir);
-	error (0, 0, "`..'-relative repositories are not supported.");
-	error (1, 0, "illegal source repository");
-    }
     if (! isabsolute(repos))
     {
 	char *newrepos;
 
-	if (CVSroot_original == NULL)
+	if (current_parsed_root == NULL)
 	{
 	    error (0, 0, "in directory %s:", xupdate_dir);
 	    error (0, 0, "must set the CVSROOT environment variable\n");
 	    error (0, 0, "or specify the '-d' option to %s.", program_name);
 	    error (1, 0, "illegal repository setting");
 	}
-	newrepos = xmalloc (strlen (CVSroot_directory) + strlen (repos) + 10);
-	(void) sprintf (newrepos, "%s/%s", CVSroot_directory, repos);
+	if (pathname_levels (repos) > 0)
+	{
+	    error (0, 0, "in directory %s:", xupdate_dir);
+	    error (0, 0, "`..'-relative repositories are not supported.");
+	    error (1, 0, "illegal source repository");
+	}
+	newrepos = xmalloc (strlen (current_parsed_root->directory) + strlen (repos) + 2);
+	(void) sprintf (newrepos, "%s/%s", current_parsed_root->directory, repos);
 	free (repos);
 	repos = newrepos;
     }
@@ -147,10 +147,10 @@ Short_Repository (repository)
 
     /* If repository matches CVSroot at the beginning, strip off CVSroot */
     /* And skip leading '/' in rep, in case CVSroot ended with '/'. */
-    if (strncmp (CVSroot_directory, repository,
-		 strlen (CVSroot_directory)) == 0)
+    if (strncmp (current_parsed_root->directory, repository,
+		 strlen (current_parsed_root->directory)) == 0)
     {
-	char *rep = repository + strlen (CVSroot_directory);
+	char *rep = repository + strlen (current_parsed_root->directory);
 	return (*rep == '/') ? rep+1 : rep;
     }
     else
